#conding=utf8
import os
import html
import numpy as np
import pandas as pd
import jieba
from dataPreprocess import checkFormat
import pickle
import re
from keras.models import Sequential, load_model
from keras.layers import Dense, Activation, Dropout, Embedding
from keras.layers import LSTM, GRU

MaxLength = 300


def readnpy(pathList):
	data = []
	for path in pathList:
		file = np.load(path)
		data.append(file)
	return data


def word2representation(path):
	file = open(path, 'rb')
	train, test = pickle.load(file)
	trainText = train['text'].to_list()
	testText = test['text'].to_list()
	trainLabel = train['label'].to_list()
	trainLabel = np.array([[int(i)] for i in trainLabel])
	testLabel = test['label'].to_list()
	testLabel = np.array([[int(i)] for i in testLabel])
	wordList = []

	for text in trainText:
		# 过滤特殊字符
		text = re.sub('[a-zA-Z0-9’!"#$%&\'()*+,-./:;<=>?@，。?★、…【】《》？“”‘’！[\\]^_`{|}~\s]+', "", text)
		# 结巴分词
		text = list(jieba.cut(text))
		wordList.extend(text)
	# 汉字->词频
	wordFrequency = pd.Series(wordList).value_counts()
	# 汉字集合
	vocab = wordFrequency.index.to_list()
	# 结尾添加空字符，对应padding下标
	vocab.append('')
	# 集合大小
	vocabSize = len(vocab)+1
	# 汉字->index
	wordDict = dict(zip(vocab, [i for i in range(1, len(vocab)+1)]))

	# 文本转换为list
	def word2num(doc, maxLength=MaxLength):
		num = []
		for text in doc:
			text = re.sub('[a-zA-Z0-9’!"#$%&\'()*+,-./:;<=>?@，。?★、…【】《》？“”‘’！[\\]^_`{|}~\s]+', "", text)
			text = list(jieba.cut(text))
			text = [i for i in text if i in vocab]
			# 转换为恒定长度list
			if len(text) < maxLength:
				# padding补全
				text = text[:maxLength] + ['']*(maxLength-len(text))
			else:
				text = text[:maxLength]
			num.append([wordDict[i] for i in text])
		return num

	trainList = np.array(word2num(trainText))
	testList = np.array(word2num(testText))
	return trainList, testList, trainLabel, testLabel, vocabSize


def rnn(pattern, trainData, trainLabel, testData, testLabel, inputDim, maxLength=MaxLength):
    model = Sequential()
    model.add(Embedding(inputDim, 256, input_length=maxLength))
    if pattern == 0:
    	model.add(LSTM(128))
    else:
    	model.add(GRU(128))
    model.add(Dropout(0.5))
    model.add(Dense(1))
    model.add(Activation('sigmoid'))
    model.compile(
        loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
    model.summary()
    model.fit(
        trainData[0: int(len(trainData)*0.8)],
        trainLabel[0: int(len(trainLabel)*0.8)],
        batch_size=128,
        nb_epoch=30,
        validation_data=(trainData[int(len(trainData)*0.8):], trainLabel[int(len(trainLabel)*0.8):])
        )

    score = model.evaluate(testData, testLabel, batch_size=128)
    print(score)
    # model.save('model_' + language + '.h5')


if __name__ == "__main__":
	trainList, testList, trainLabel, testLabel, vocabSize = word2representation('eventPick.pkl')
	# print(trainList.shape, trainLabel.shape, testList.shape, testLabel.shape)
	# pathList = ["w2vTrain.npy","w2vTest.npy","w2vTrainLabel.npy","w2vTestLabel.npy","w2vVocabSize.npy"]
	# pathList = ["preprocessResult/"+i for i in pathList]
	# data = readnpy(pathList)
	# trainList, testList, trainLabel, testLabel, vocabSize = data[0],data[1],data[2],data[3], int(data[4])
	rnn( 0, trainList, trainLabel, testList, testLabel, vocabSize)
# checkFormat("weiboText.pkl")