#conding=utf8
import os
import html
import numpy as np
import pandas as pd
import jieba
from dataPreprocess import checkFormat
import pickle
import re
from keras.models import Sequential, load_model
from keras.layers import Dense, Activation, Dropout, Embedding, BatchNormalization
from keras.layers import LSTM, GRU
from keras.layers.core import Masking
from attention import AttentionWithContext
MaxLength = 80
MaxTimeLength = 10
totalLen = MaxLength+2*MaxTimeLength


def word2representation(path):
	file = open(path, 'rb')
	train, test = pickle.load(file)
	trainText = train['text'].to_list()
	trainTimeAvg = train['timeAvg'].to_list()
	trainTimeVar = train['timeVar'].to_list()
	testText = test['text'].to_list()
	testTimeAvg = test['timeAvg'].to_list()
	testTimeVar = test['timeVar'].to_list()
	trainLabel = train['label'].to_list()
	trainLabel = np.array([[int(i)] for i in trainLabel])
	testLabel = test['label'].to_list()
	testLabel = np.array([[int(i)] for i in testLabel])

	wordList = []
	count = 0
	for text in trainText:
		# 过滤特殊字符
		text = re.sub('[a-zA-Z0-9’!"#$%&\'()*+,-./:;<=>?@，。?★、…【】《》？“”‘’！[\\]^_`{|}~\s]+', "", text)
		# 结巴分词
		text = list(jieba.cut(text))
		wordList.extend(text)
		count+=1
		print(count)
	# 汉字->词频
	wordFrequency = pd.Series(wordList).value_counts()
	# 汉字集合
	vocab = wordFrequency.index.to_list()
	# 结尾添加空字符，对应padding下标
	vocab.append('')
	# 集合大小
	vocabSize = len(vocab)+1
	# 汉字->index
	wordDict = dict(zip(vocab, [i for i in range(1, len(vocab)+1)]))


	# 文本转换为list
	def word2num(doc, maxLength=MaxLength):
		num = []
		for text in doc:
			text = re.sub('[a-zA-Z0-9’!"#$%&\'()*+,-./:;<=>?@，。?★、…【】《》？“”‘’！[\\]^_`{|}~\s]+', "", text)
			text = list(jieba.cut(text))
			text = [i for i in text if i in vocab]
			# 转换为恒定长度list
			if len(text) < maxLength:
				# padding补全
				text = text[:maxLength] + ['']*(maxLength-len(text))
			else:
				text = text[:maxLength]
			num.append([wordDict[i] for i in text])
		return num


	def paddingTime(timeList):
		newTimeList = []
		for time in timeList:
			# 转换为恒定长度list
			if len(time)<MaxTimeLength:
				# padding补全
				time = time+[0]*(MaxTimeLength-len(time))
			else:
				time = time[:MaxTimeLength]
			newTimeList.append(time)
		return newTimeList


	trainList = word2num(trainText)
	testList = word2num(testText)
	trainTimeAvg = paddingTime(trainTimeAvg)
	trainTimeVar = paddingTime(trainTimeVar)
	testTimeAvg = paddingTime(testTimeAvg)
	testTimeVar = paddingTime(testTimeVar)
	for i in range(len(trainList)):
		tmp = trainList[i]
		tmp.extend(trainTimeAvg[i])
		tmp.extend(trainTimeVar[i])
		trainList[i] = tmp
	for i in range(len(testList)):
		tmp = testList[i]
		tmp.extend(testTimeAvg[i])
		tmp.extend(testTimeVar[i])
		testList[i] = tmp
	trainList = np.array(trainList)
	testList = np.array(testList)
	# print("SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS\n",trainList)
	np.save("eventTrain_v2.npy", trainList)
	np.save("eventTest_v2.npy", testList)
	# np.save("eventTrainTimeAvg_v2.npy", trainTimeAvg)
	# np.save("eventTestTimeAvg_v2.npy", testTimeAvg)
	# np.save("eventTrainTimeVar_v2.npy", trainTimeVar)
	# np.save("eventTestTimeVar_v2.npy", testTimeVar)
	np.save("eventTrainLabel_v2.npy", trainLabel)
	np.save("eventTestLabel_v2.npy", testLabel)
	np.save('eventVocab_v2', np.array(vocabSize))
	return trainList, testList, trainLabel, testLabel, vocabSize


def readnpy2(pathList):
	data = []
	for path in pathList:
		file = np.load(path)
		flag=True
		for t in file:
			if flag==True:
					for tt in t:
						if tt > vocabSize or tt<0:
							print(t)
						flag=False
						break
		data.append(file)
	return data


def readnpy(pathList):
	data = []
	for path in pathList:
		file = np.load(path)
		data.append(file)
	return data


def rnn(pattern, trainData, trainLabel, testData, testLabel, inputDim, epoch, maxLength=MaxLength):
    model = Sequential()
    # model.add(Masking(mask_value=0., input_shape=shape[1:]))
    model.add(Embedding(inputDim, 256, input_length=maxLength))
    if pattern == 0:
    	model.add(LSTM(128))
    	# model.add(AttentionWithContext())
		# model.add()
    else:
    	model.add(GRU(128))
    model.add(Dropout(0.5))
    model.add(Dense(1))
    model.add(Activation('sigmoid'))
    model.compile(
        loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
    model.summary()
    model.fit(
        trainData[0: int(len(trainData)*0.8)],
        trainLabel[0: int(len(trainLabel)*0.8)],
        batch_size=128,
        nb_epoch=epoch,
        validation_data=(trainData[int(len(trainData)*0.8):], trainLabel[int(len(trainLabel)*0.8):])
        )

    score = model.evaluate(testData, testLabel, batch_size=128)
    print("********************************************* ", vocabSize)
    print(score)
    # model.save('model_' + language + '.h5')


if __name__ == "__main__":
	# path = 'eventPickle.pkl'
	# path = 'weiboFullText.pkl'
	# path = 'weiboText.pkl'
	# path = 'eventPickle_v3.pkl'
	# vocabSize=vocab(path)
	vocabSize=347909 # for fulltext
	# vocabSize = int(np.load('eventVocab_v2.npy'))
	# pathList = ['eventTrain_v2.npy','eventTest_v2.npy']
	# trainList, testList = readnpy(pathList)
	pathList = ['eventTrain_v2.npy','eventTest_v2.npy','eventTrainLabel_v2.npy','eventTestLabel_v2.npy','eventVocab_v2.npy']
	pathList = ['weiboFullTrain.npy','weiboFullTest.npy','weiboFullTrainLabel.npy','weiboFullTestLabel.npy']
	data = readnpy(pathList)
	trainList, testList, trainLabel, testLabel = data[0], data[1], data[2], data[3]
	# trainList=np.nan_to_num(trainList)
	# testList=np.nan_to_num(testList)
	# data = readnpy(['weiboFullTrain.npy','weiboFullTest.npy','weiboFullTrainLabel.npy','weiboFullTestLabel.npy'])
	# trainList, testList, trainLabel, testLabel = data[0], data[1], data[2], data[3]
	rnn( pattern=0, trainData=trainList, trainLabel=trainLabel, testData=testList, testLabel=testLabel, inputDim=vocabSize, epoch=5, maxLength=MaxLength)
# checkFormat("weiboText.pkl")